/**************************************************************
LED PWM Program (3 color led cathode common  use)
 **************************************************************
	author air_variable
	http://www.ne.jp/asahi/air/variable/
	Tools:
	MPLAB v8.84
	Hitech PIC C pro9.83/ microchip XC8 compiler 1.00
---------------------------------------------------------------
Date	:Time	Version	Build	Note
20040102 ----	0.30	9.60	Release version
20120406 0750	1.01	9.83	corresponding new compiler 
20120406 1000	1.02	9.83	change Global parameter to Local parameter 

---------------------------------------------------------------
port assignment
1	VDD					8	VSS
2	GP5	(not used)		7	GP0	O	LED red
3	GP4	(not used)		6	GP1	O	LED green
4	GP3	I VCC or Pu		5	GP2	O	LED blue

VDD 2.2V to 5V
GP3 is MUST do PULLUP 10K OHM,if you use Pickit2/ICD2 is active.

*/
#include <htc.h>
#define MHz 000000
#define _XTAL_FREQ 4MHz

//----------------------------------------------
#define PwmResolution 127
#define PwmCycleSpan 8
#define PauseSpeed 200	//x 10millisecond
//----------------------------------------------

//LED port define
#define PortLED0 GP0
#define PortLED1 GP1
#define PortLED2 GP2

//start state set
#define StartState 0

//pause setting
#define PauseColor delay_10ms(PauseSpeed);

//LED polarity setting. Cathode common
#define off 0
#define on 1


//Chip configuration
__CONFIG(CP_OFF & BOREN_OFF & MCLRE_OFF & PWRTE_OFF & WDTE_OFF & FOSC_INTRCIO);	//Idoling current 0.06uA

//10millisecond timer
void delay_10ms(unsigned char temp){
	while(temp--){
		__delay_ms(10);
	}	
}


main(void){
//Local parameter setting
	unsigned char outcnt,i;
	unsigned char PWM_LED0,PWM_LED1,PWM_LED2,state;

//Initialization
    CMCON  = 7;
#if defined(_12F675)
	//digital port set
    ANSEL  = 0b00000000;
#endif
    TRISIO = 0b11111000;//gp2 is output

//PWM LED data set
	PWM_LED0 = PwmResolution;
	PWM_LED1 = PWM_LED2 = state = 0;	
		
	while (1){
		outcnt= PwmCycleSpan;
		while(outcnt--){
			unsigned char i;
			for (i=1; i!=PwmResolution ;  i++){
				if ( PWM_LED0 <= i ) PortLED0=off; else PortLED0=on;
				if ( PWM_LED1 <= i ) PortLED1=off; else PortLED1=on;
				if ( PWM_LED2 <= i ) PortLED2=off; else PortLED2=on;
			}
		}	

		switch (state){	
			case 0:PWM_LED1++;//001->011
				if (PWM_LED1 > PwmResolution ) {
					state=1;
					PauseColor;
				}
				break;
			case 1:PWM_LED0--;//011->010
				if (PWM_LED0 ==0) {
					state=2;
					PauseColor;
				}
				break;
			case 2:PWM_LED2++;//010->110
				if (PWM_LED2 > PwmResolution ) {
				 state=3;
					PauseColor;
				}
				break;
			case 3:PWM_LED1--;//110->100
				if (PWM_LED1 ==0) {
					state=4;
					PauseColor;
				}
				break;
			case 4:PWM_LED0++;//100->101
				if (PWM_LED0 > PwmResolution ) {
					state=5;
					PauseColor;
				}
				break;
			case 5:PWM_LED2--;//101->001
				if (PWM_LED2 ==0) {
					state=6;
					PauseColor;
				}
				break;

			default:
				state=0;
			
		}
    }
}
